# -*- coding: utf-8 -*-
'''
    pygeodiff
    -----------
    This module provides tools for create diffs of geospatial data formats
    :copyright: (c) 2019 Peter Petrik
    :license: MIT, see LICENSE for more details.
'''

from .main import GeoDiff
from .geodifflib import GeoDiffLibError, GeoDiffLibConflictError



# -*- coding: utf-8 -*-
'''
    pygeodiff.main
    --------------
    Main entry of the library
    :copyright: (c) 2019 Peter Petrik
    :license: MIT, see LICENSE for more details.
'''

from .geodifflib import GeoDiffLib


class GeoDiff:
    """
        if libname is None, it tries to import c-extension from wheel
    """
    def __init__(self, libname=None):
        self.clib = GeoDiffLib(libname)

    """
        Creates changeset file (binary) in such way that
        if CHANGESET is applied to BASE by applyChangeset,
        MODIFIED will be created
 
        BASE --- CHANGESET ---> MODIFIED
 
        \param base [input] BASE sqlite3/geopackage file
        \param modified [input] MODIFIED sqlite3/geopackage file
        \param changeset [output] changeset between BASE -> MODIFIED
        
        raises SqliteDiffError on error
    """
    def create_changeset(self, base, modified, changeset):
        return self.clib.create_changeset(base, modified, changeset)

    """ 
        Creates changeset file (binary) in such way that
        if CHANGESET is applied to MODIFIED_THEIR by
        applyChangeset, the new state will contain all
        changes from MODIFIED and MODIFIED_THEIR.
        
               --- CHANGESET_THEIR ---> MODIFIED_THEIR --- CHANGESET ---> MODIFIED_THEIR_PLUS_MINE
        BASE -| 
               -----------------------> MODIFIED
        
         \param base [input] BASE sqlite3/geopackage file
         \param modified [input] MODIFIED sqlite3/geopackage file
         \param changeset_their [input] changeset between BASE -> MODIFIED_THEIR
         \param changeset [output] changeset between MODIFIED_THEIR -> MODIFIED_THEIR_PLUS_MINE
         
         raises SqliteDiffError on error
    """
    def create_rebased_changeset(self, base, modified, changeset_their, changeset):
        return self.clib.create_rebased_changeset(base, modified, changeset_their, changeset)

    """
        Applies changeset file (binary) to BASE and creates PATCHED.
        
        \param base [input] BASE sqlite3/geopackage file
        \param patched [output] PATCHED sqlite3/geopackage file with changeset
        \param changeset [input] changeset between BASE -> PATCHED
        \returns number of conflics
        
        raises SqliteDiffError on error
    """
    def apply_changeset(self, base, patched, changeset):
        return self.clib.apply_changeset(base, patched, changeset)

    """ 
        Lists changeset content to stdout
        \returns number of changes
         
         reises SqliteDiffError on error
    """
    def list_changes(self, changeset):
        return self.clib.list_changes(changeset)

    def version(self):
        return self.clib.version()


def main():
    diff_lib = GeoDiff()
    print("pygeodiff " + diff_lib.version())



# -*- coding: utf-8 -*-
'''
    pygeodiff.geodifflib
    --------------------
    This module provides wrapper of geodiff C library
    :copyright: (c) 2019 Peter Petrik
    :license: MIT, see LICENSE for more details.
'''

import ctypes
import os
import platform
from ctypes.util import find_library
from .__about__ import __version__


class GeoDiffLibError(Exception):
    pass


class GeoDiffLibConflictError(GeoDiffLibError):
    pass


class GeoDiffLibVersionError(GeoDiffLibError):
    pass


# keep in sync with c-library
SUCCESS=0
ERROR=1
CONFLICT=2


def _parse_return_code(rc, msg):
    if rc == SUCCESS:
        return
    elif rc == ERROR:
        raise GeoDiffLibError(msg)
    elif rc == CONFLICT:
        raise GeoDiffLibConflictError(msg)


class GeoDiffLib:
    def __init__(self, name):
        if name is None:
            # ok lets assume that the package is installed through PIP
            if platform.system() == 'Windows':
                prefix = ""
                suffix = ".dll"
            elif platform.system() == 'Darwin':
                prefix = "lib"
                suffix = ".dylib"
            else:
                prefix = "lib"
                suffix = ".so"
            whl_lib = prefix + 'pygeodiff-' + __version__ + '-python' + suffix
            dir_path = os.path.dirname(os.path.realpath(__file__))
            self.libname = os.path.join(dir_path, whl_lib)
            if not os.path.exists(self.libname):
                # not found, try system library
                self.libname = find_library("geodiff")
        else:
            self.libname = name

        if self.libname is None:
            raise GeoDiffLibVersionError("Unable to locate geodiff library")

        self.lib = ctypes.CDLL(self.libname, use_errno=True)
        self.init()
        self.check_version()

    def init(self):
        func = self.lib.GEODIFF_init
        func()

    def version(self):
        func = self.lib.GEODIFF_version
        func.restype = ctypes.c_char_p
        ver = func()
        return ver.decode('utf-8')

    def check_version(self):
        cversion = self.version()
        pyversion = __version__
        if cversion != pyversion:
            raise GeoDiffLibVersionError("version mismatch (%s C vs %s PY)".format(cversion, pyversion))

    def create_changeset(self, base, modified, changeset):
        func = self.lib.GEODIFF_createChangeset
        func.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
        func.restype = ctypes.c_int

        # create byte objects from the strings
        b_string1 = base.encode('utf-8')
        b_string2 = modified.encode('utf-8')
        b_string3 = changeset.encode('utf-8')

        res = func(b_string1, b_string2, b_string3)
        _parse_return_code(res, "createChangeset")

    def create_rebased_changeset(self, base, modified, changeset_their, changeset):
        func = self.lib.GEODIFF_createRebasedChangeset
        func.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
        func.restype = ctypes.c_int

        # create byte objects from the strings
        b_string1 = base.encode('utf-8')
        b_string2 = modified.encode('utf-8')
        b_string3 = changeset_their.encode('utf-8')
        b_string4 = changeset.encode('utf-8')

        res = func(b_string1, b_string2, b_string3, b_string4)
        _parse_return_code(res, "createRebasedChangeset")

    def apply_changeset(self, base, patched, changeset):
        func = self.lib.GEODIFF_applyChangeset
        func.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
        func.restype = ctypes.c_int

        # create byte objects from the strings
        b_string1 = base.encode('utf-8')
        b_string2 = patched.encode('utf-8')
        b_string3 = changeset.encode('utf-8')

        res = func(b_string1, b_string2, b_string3)
        _parse_return_code(res, "createRebasedChangeset")

    def list_changes(self, base ):
        func = self.lib.GEODIFF_listChanges
        func.argtypes = [ctypes.c_char_p]
        func.restype = ctypes.c_int

        # create byte objects from the strings
        b_string1 = base.encode('utf-8')
        nchanges = func(b_string1)
        if nchanges < 0:
            raise GeoDiffLibError("listChanges")
        return nchanges



__title__ = 'PyGeoDiff'
__description__ = 'Diff tool for geo-spatial data'
__url__ = 'https://github.com/lutraconsulting/geodiff'
__version__ = '0.3.0'
__author__ = 'Peter Petrik'
__author_email__ = 'zilolv@gmail.com'
__maintainer__ = 'Peter Petrik'
__license__ = 'MIT'
__copyright__ = '(c) 2019 Peter Petrik'